import java.util.Arrays;
import java.util.Comparator;
import java.util.Random;

public class Flight {

    private String destination;
    private BoardingQueue  boardQueue;
    private Passenger[] firstClass = new Passenger[2];
    private Passenger[] businessClass= new Passenger[3];
    private Passenger[] premiumEconomy= new Passenger[4];
    private Passenger[] economy= new Passenger[6];
    private int minutesLeftBoarding;
    private int minutesLeftDeparting;
    private int firstClassInd;
    private int businessClassInd;
    private int premiumEconomyInd;
    private int economyInd;
    private boolean boarding;
    private int boarded;
    private int passengerID;
    private FlightType flightType;
    private final Comparator<Passenger> comparator= new SortByTime();

    public Flight(String destination) {
        this();
        this.destination = destination;
    }

    public Flight() {
        this.boardQueue = new BoardingQueue();
        this.minutesLeftBoarding = 25;
        this.minutesLeftDeparting = 5;
        this.firstClassInd = 0;
        this.businessClassInd = 0;
        this.premiumEconomyInd = 0;
        this.economyInd = 0;
        this.boarding = false;
        this.boarded=0;
        passengerID=1;
    }

    public void addToFlight(Passenger boardedPass) {
        switch(boardedPass.getPassClass()){
            case FIRST_CLASS:
                boardFirstClass(boardedPass);
                break;
            case BUSINESS_CLASS:
                boardBusiness(boardedPass);
                break;
            case PREMIUM_ECONOMY:
                boardPremium(boardedPass);
                break;
            case ECONOMY:
                boardEconomy(boardedPass);
                break;
        }
    }

    private void boardMessage(Passenger passenger, String seat){
        System.out.println("-"+passenger.getPassClass().name+ " Passenger (ID "+passenger.getPassengerID()+") on flight to "+destination+" has boarded on a "+seat+" seat!");
    }
    private void boardFirstClass(Passenger passenger){
        if (this.firstClassInd <2) {
            this.firstClass[this.firstClassInd++] = passenger;
            boardMessage(passenger, "First-class");
            boarded++;
        }
        else
            boardBusiness(passenger);
    }

    private void boardBusiness(Passenger passenger){
        if (this.businessClassInd < 3) {
            this.businessClass[businessClassInd++] = passenger;
            boardMessage(passenger, "Business-class");
            boarded++;
        }
        else
            boardPremium(passenger);
    }
    private void boardPremium(Passenger passenger){
        if (this.premiumEconomyInd < 4) {
            this.premiumEconomy[this.premiumEconomyInd++] = passenger;
            boardMessage(passenger, "Premium-economy");
            boarded++;
        }
        else
            boardEconomy(passenger);
    }
    private void boardEconomy(Passenger passenger){
        if (this.economyInd < 6) {
            this.economy[economyInd++] = passenger;
            boardMessage(passenger, "Economy");
            boarded++;
        }
    }

    public boolean isFull(){
        return this.boarded == 15;
    }

    public String getDestination() {
        return destination;
    }

    public BoardingQueue getBoardQueue() {
        return boardQueue;
    }

    public int getMinutesLeftBoarding() {
        return minutesLeftBoarding;
    }

    public int getMinutesLeftDeparting() {
        return minutesLeftDeparting;
    }

    public boolean isBoarding() {
        return boarding;
    }

    public void setDestination(String destination) {
        this.destination = destination;
    }

    public void setMinutesLeftBoarding() {
        if (this.minutesLeftBoarding > 0)
        this.minutesLeftBoarding -= 1;
    }

    public void setMinutesLeftDeparting() {
        if (this.minutesLeftDeparting > 0)
        this.minutesLeftDeparting -= 1;
    }

    public FlightType getFlightType() {
        return flightType;
    }

    public void setFlightType(Random random) {
        this.flightType = random.nextInt(20)+1 == 1? FlightType.AIR_FORCE: FlightType.NORMAL;
    }

    public void setBoarding(boolean boarding) {
        this.boarding = boarding;
    }


    public boolean addPassengerToQueue(int arrivalTime,  float probability, Random randomNumberGenerator) {
        float chance= (float) (Math.round(randomNumberGenerator.nextFloat()*10)/10.0);
        if(chance <= probability){
            //Create a passenger
            TravelClass travelClass= chooseClass(randomNumberGenerator);
            Passenger passenger = new Passenger(travelClass, passengerID++, arrivalTime);
            passenger.setCovidPositive(randomNumberGenerator);

            //board passenger
            try {
                boardQueue.enqueuePassenger(passenger);
                System.out.println("-"+travelClass.name+" passenger(ID "+passenger.getPassengerID()+") on flight to "+destination+" has entered flights boarding queue");
            } catch (NoRoomException e) {
                System.out.println("-Flight to "+this.destination+": "+e.getMessage());
                return false;
            }

            return true;
        }

        return false;
    }

    public boolean boardPassengers(float probability, Random randomNumberGenerator){
        if(isFull())
            return false;

        float chance= (float) (Math.round(randomNumberGenerator.nextFloat()*10)/10.0);
        if(chance <= probability){

            try {
                Passenger passenger= boardQueue.dequeuePassenger();
                if (passenger.isCovidPositive()){
                    minutesLeftBoarding+=10;
                    System.out.println("-Passenger (ID "+passenger.getPassengerID()+") on flight to "+destination+"has covid and delayed flight by 10 more minutes");
                    return true;
                }

                addToFlight(passenger);
                return true;
            } catch (NoPassengerException e) {
                System.out.println("-Flight to "+destination+": "+e.getMessage());
                return false;
            }
        }

        return false;
    }
    private TravelClass chooseClass(Random randomNumberGenerator){
        switch (randomNumberGenerator.nextInt(4)+1){
            case 1: return TravelClass.FIRST_CLASS;
            case 2: return TravelClass.BUSINESS_CLASS;
            case 3: return TravelClass.PREMIUM_ECONOMY;
            default: return TravelClass.ECONOMY;
        }
    }

    public void boardingMessage(){
        System.out.println("-Flight " +destination+" has "+minutesLeftBoarding+" minutes to board, "+boarded+" passenger(s), and "+boardQueue.getSize()+" person(s) waiting to board.");
    }

    public boolean canDepart() {
        return minutesLeftBoarding == 0 || boarded == 15;
    }

    public boolean finalDeparture() {
        return minutesLeftDeparting == 0;
    }

    public void departingMessage(){
        System.out.println("-Flight " +destination+" has "+boarded+" passengers  and is "+minutesLeftDeparting+" minute away from departure.");
    }

    public void finalDepartureMessage(){

        System.out.format("Flight: RFK -> %s %n", destination);
        System.out.format("| %-20s    | %-20s   | %-20s %n", "Seat Type", "ID", "Arrival Time");
        System.out.format(" ------------------------------------------------------------------------------ %n");

        for (Passenger passenger: firstClass)
            printPassengerData(passenger, "First Class");

        for (Passenger passenger: businessClass)
            printPassengerData(passenger, "Business Class");

        for (Passenger passenger: premiumEconomy)
            printPassengerData(passenger, "Premium Economy");

        for (Passenger passenger: economy)
            printPassengerData(passenger, "Economy");
    }

    private  void printPassengerData(Passenger passenger, String seat){
        if (passenger != null) {
            String format = "| %-23s | %-22d | %-22d |%n";
            System.out.format(format, seat, passenger.getPassengerID(), passenger.getArrivalTime());
        }
    }
}
