import java.time.Instant;
import java.util.Random;
import java.util.Scanner;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import java.util.logging.Logger;

public class FlightSimulator {

    private final Flight[] flightTerminals = new Flight[20];
    private int currentFlights=0;

    private final String[] destinations = {"SJC", "LAX"};
    private static Random randomNumberGenerator;

    //Probabilities
    private static float PASSENGER_ARRIVAL;
    private static float PASSENGER_DEQUEUED;
    private static float NEW_FLIGHT;

    private static int MINUTES_USED;


    private boolean createFlight(){
        if (currentFlights < 20) {
            currentFlights++;
            float chance = (float) (Math.round(randomNumberGenerator.nextFloat() * 10) / 10.0);

            if (chance <= NEW_FLIGHT) {
                int dest = randomNumberGenerator.nextInt(destinations.length);
                Flight flight = new Flight(destinations[dest]);
                flight.setBoarding(true);
                flight.setFlightType(randomNumberGenerator);

                System.out.println("- A new flight to..." + flight.getDestination() + "...has begun boarding");
                for (int i = 0; i < flightTerminals.length; i++) {
                    if (flightTerminals[i] == null) {
                        flightTerminals[i] = flight;
                        break;
                    }
                }

            }

            return true;
        }
        return false;
    }

    private boolean queuePassengers(int time){
        boolean added = false;
        for (Flight flight : flightTerminals) {
            if (flight != null)
                added = added || flight.addPassengerToQueue(time, PASSENGER_ARRIVAL, randomNumberGenerator);
        }

        return added;
    }

    private boolean boardPassengers(){
        boolean boarded = false;
        for (Flight flight : flightTerminals) {
            if (flight != null)
                boarded = boarded || flight.boardPassengers(PASSENGER_DEQUEUED, randomNumberGenerator);
        }

        return boarded;
    }

    private boolean events(int time) {

        boolean createFlight = createFlight();
        boolean queuePass = queuePassengers(time);
        boolean boardPassengers = boardPassengers();

        System.out.println();
        return queuePass || createFlight || boardPassengers;
    }

    private boolean boarding() {
        boolean boarding = false;
        for (Flight flightTerminal : flightTerminals) {
            if (flightTerminal != null && !flightTerminal.isFull()) {
                flightTerminal.boardingMessage();
                flightTerminal.setMinutesLeftBoarding();
                boarding = true;
            }
        }

        return boarding;
    }

    private boolean departing() {
        boolean departing = false;
        for (Flight flightTerminal : flightTerminals) {
            if (flightTerminal != null && flightTerminal.canDepart()) {
                flightTerminal.departingMessage();
                flightTerminal.setMinutesLeftDeparting();
                departing = true;
            }
        }

        return departing;
    }

    private boolean finalDepartures(){
        boolean finalDeparture = false;
        for (int i = 0; i < flightTerminals.length; i++) {
            if (flightTerminals[i] != null && flightTerminals[i].finalDeparture()) {
                flightTerminals[i].finalDepartureMessage();
                flightTerminals[i] = null;
                finalDeparture = true;
            }
        }

        return finalDeparture;
    }

    private static float readProbabilities(String prompt, Scanner scanner){
        System.out.print(prompt);
        return scanner.nextFloat();
    }

    private void simulate(){
        System.out.println("Minute: "+MINUTES_USED+"\n" +
                "*******************************************************************");
        System.out.println("\nEvents: ");
        if(!events(MINUTES_USED++))
            System.out.println("Nothing to note...\n");

        System.out.println("\nBoarding");
        if(!boarding())
            System.out.println("Nothing to note...\n");

        System.out.println("\nDeparting:");
        if(!departing())
            System.out.println("\nNothing to note...\n");

        System.out.println("\nFinal Departures: ");
        if(!finalDepartures())
            System.out.println("Nothing to note...\n");

        System.out.println( "*******************************************************************");
    }

    public static void main(String[] args) throws InterruptedException {
        Scanner scanner= new Scanner(System.in);
        System.out.println("Welcome to RFK");

        System.out.print("Enter seed for this simulator: ");
        long seed = scanner.nextLong();

        PASSENGER_ARRIVAL = readProbabilities("Enter the probability of a passenger arrival: ", scanner);
        PASSENGER_DEQUEUED = readProbabilities("Enter the probability that a passenger is dequeued: ", scanner);
        NEW_FLIGHT = readProbabilities("Enter the probability that there is a new flight at RFK: ", scanner);

        System.out.print("Enter how many minutes this simulation should take: ");
        //Minutes taken
        int MINUTES = scanner.nextInt();
        System.out.println();

        randomNumberGenerator = new Random();
        FlightSimulator simulator= new FlightSimulator();

        ScheduledExecutorService executor = Executors.newScheduledThreadPool ( 1 );

        Runnable r =  () -> {
            try {  // Always wrap your Runnable with a try-catch as any uncaught Exception causes the ScheduledExecutorService to silently terminate.
                simulator.simulate();

            } catch ( Exception e ) {
                e.printStackTrace();
                System.out.println ( "Error occurred" );
                executor.shutdownNow();
            }
        };

        try {
            executor.scheduleAtFixedRate ( r , 0L , 1L , TimeUnit.MINUTES ); // ( runnable , initialDelay , period , TimeUnit )
            Thread.sleep ( TimeUnit.MINUTES.toMillis (MINUTES) ); // Let things run a minute to witness the background thread working.
        } catch ( InterruptedException ex ) {
            Logger.getLogger ( FlightSimulator.class.getName () ).log ( Level.SEVERE , null , ex );
        } finally {
            System.out.println ( "Simulation Ended" );
            executor.shutdown ();
        }


//        Thread.sleep(1000)
    }
}
